#	Masrv
#	Copyright (C) 2007 Michal Vaner (vorner@ucw.cz)
#
#	This program is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	he Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import glob
import config
from PIL import Image

class im:
	"""Internal object for holding the images"""
	def __init__( self, image, name ):
		self.image = image
		self.score = 0
		self.name = name

class images:
	"""The image cache and manipulator.
	Ask this thing for the sizes or data of an image"""
	def __init__( self ):
		glob.info( "Starting the image storage" )
		self.__cache = {}
		self.__maxcache = int( config.data.get( "maximgcache", 100 ) )
		self.__lowcache = int( config.data.get( "lowimgcache", 50 ) )

	def __clean_cache( self ):
		glob.info( "Cleaning image cache" )
		cache = self.__cache.values()
		cache.sort( lambda x, y: cmp( y.score, x.score ) )
		cache = cache[ 0:self.__lowcache ]
		self.__cache = {}
		for i in cache:
			self.__cache[ i.name ] = i
			i.score = 0

	def __get_img( self, imgname ):
		if not imgname in self.__cache:
			glob.info( "Image not in cache, loading" )
			nimg = im( Image.open( imgname ), imgname )
			if len( self.__cache ) >= self.__maxcache:
				self.__clean_cache()
			self.__cache[ imgname ] = nimg
		result = self.__cache[ imgname ]
		result.score += 1
		return result

	def size( self, imgname ):
		return self.__get_img( imgname ).image.size

	def data( self, imgname, flipx, flipy ):
		img = self.__get_img( imgname ).image
		if flipx: img = img.transpose( Image.FLIP_LEFT_RIGHT )
		if flipy: img = img.transpose( Image.FLIP_TOP_BOTTOM )
		return img.convert( "RGBA" ).tostring()
